
/******************************************************************************
* MODULE     : complexify_glue.hpp
* DESCRIPTION: Glue for complexified numbers
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_COMPLEXIFY_GLUE_HPP
#define __MMX_COMPLEXIFY_GLUE_HPP
#include "complexify.hpp"
#include "category_meta.hpp"
#include "category_glue.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Complexify complexify<C>

template<typename T,typename F>
struct cast_helper<complexify<T>,complexify<F> > {
  static inline complexify<T> cv (const complexify<F>& z) {
    return complexify<T> (as<T> (Re (z)), as<T> (Im (z))); }
};

template<typename C> Complexify
construct_complexify (const C& x, const C& y) {
  return Complexify (x, y);
}

template<typename Hyp, typename C> void
define_complexify () {
  typedef both<has<C,ring_cat>,Hyp> Cond;
  typedef both<different<C,generic>,Cond> Specialized;
  define_type<Cond,Complexify > (gen ("Complexify", type_name<C> ()));
  define<Cond> ("complex", construct_complexify<C>);
  define_upgrader<Specialized, Complexify,C> (generic ("C->Complexify(C)"));
  define_ring<Cond,Complexify> ();
  define_field<both<has<C,field_cat>,Cond>, Complexify> ();
  define_elementary<both<has<C,elementary_cat>, Specialized>, Complexify> ();
}

#undef TMPL
#undef Complexify
} // namespace mmx
#endif // __MMX_COMPLEXIFY_GLUE_HPP
