
/******************************************************************************
* MODULE     : modular.hpp
* DESCRIPTION: modulo p arithmetic
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

// The implementation is intended to be portable for any p that fits a nat,
// unless "long long unsigned int" ==  "long unsigned int"
// (which is not tested).

// FIXME: The modular type should later become of the form modular<C,p>,
// where C is a constant type with division and p an element of C
// FIXME: We currently const-ed all operations on modular integers
// for higher efficiency, this should be avoided
// FIXME: We only made a very crude implementation;
// the implementations of the arithmetic operations are far from optimal

#ifndef __MMX__MODULAR__HPP
#define __MMX__MODULAR__HPP
#include "basix.hpp"

namespace mmx {
#define TMPL template<class P>
#define Modular modular<P>

typedef long long int long_int;
typedef long long unsigned int long_nat;

nat invert_modulo (nat i, nat p);

/******************************************************************************
* Integers modulo p
******************************************************************************/

template<typename P>
class modular {
MMX_ALLOCATORS;
public:
  nat i;
  static const nat p= P::val;

  inline modular (): i (0) {}
  inline modular (int i2):
    i (i2 >= 0? ((nat) i2)% p: p-1 - (((nat) (-1-i2))%p)) {}
  inline modular (nat i2):
    i (i2 % p) {}
  //modular (long_nat i2):
  //  i (i2 % p) {}
  Modular operator + (const Modular& a) const {
    Modular c;
    c.i = i + a.i;
    // When p is too big then we must detect overflow.
    if (p > (((nat) 1) << (8*sizeof(nat) - 1))) {
      if (c.i < i) {
	c.i -= p;
	return c;
      }
    }
    if (c.i >= p)
      c.i -= p;
    return c;
  }
  Modular operator - () const {
    Modular c;
    if (i!=0) c.i = p - i;
    return c;
  }
  Modular operator - (const Modular& a) const {
    Modular c;
    c.i = i - a.i;
    if ( i < a.i ) {
      c.i += p;
    }
    return c;
  }
  Modular operator * (const Modular& a) const {
    Modular c;
    long_nat tmp = ((long_nat) i) * a.i;
    tmp %= p;
    c.i = (nat) tmp;
    return c;
  }
  inline friend Modular invert (const Modular& j) {
    return invert_modulo (j.i, p); }
  inline Modular operator / (const Modular& j) const {
    return this->operator*(invert (j)); }
  inline bool operator == (const Modular& j) const {
    return i == j.i; }
  inline bool operator != (const Modular& j) const {
    return i != j.i; }
};

TMPL inline generic
flatten (const Modular& i) {
  return flatten (i.i);
}

PRINT_SUGAR(TMPL,Modular)

#undef TMPL
#undef Modular
} // namespace mmx
#endif //__MMX__MODULAR__HPP
