
/******************************************************************************
* MODULE     : skew_polynomial_test.cc
* DESCRIPTION: Test skew polynomials
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
#include "skew_polynomial.hpp"
//#include "skew_extra.hpp"
using namespace mmx;

#define Scalar rational
#define Polynomial \
  polynomial<Scalar,dense_polynomial<karatsuba_multiply<> > >
#define Operator xd_operator
//#define Operator S_operator<Scalar>
//Scalar Two (2);
//#define Operator q_operator<Scalar,Two>
//Polynomial X_square (Scalar (1), 3);
//#define Operator derivation_operator<Polynomial,X_square>
#define Skew_polynomial \
  polynomial<Polynomial,dense_polynomial<skew_multiply<Operator> > >

void
test (const Skew_polynomial& A, const Skew_polynomial& B) {
  Skew_polynomial Q= quo (A, B);
  Skew_polynomial R= rem (A, B);
  cout << "Test " << (A == Q*B + R) << "\n";
}

int
main () {
  Polynomial X= seq (Scalar (0), Scalar (1));
  Polynomial P= seq (Scalar (1), Scalar (5), Scalar (2));
  Polynomial Q= seq (Scalar (2), Scalar (3), Scalar (7), Scalar (3));
  Polynomial R= seq (Scalar (1), Scalar (1));
  Skew_polynomial x= seq (X);
  Skew_polynomial U= seq (Polynomial (0), Polynomial (1));
  Skew_polynomial K= seq (P, Q, R);
  Skew_polynomial L= seq (R, Polynomial (0), Q);
  cout << "x\t= " << x << "\n";
  cout << "D\t= " << U << "\n";
  cout << "K\t= " << K << "\n";
  cout << "L\t= " << L << "\n";
  cout << "K + L\t= " << K+L << "\n";
  cout << "K - L\t= " << K-L << "\n";
  cout << "L - K\t= " << L-K << "\n";
  cout << "x D\t= " << x*U << "\n";
  cout << "D x\t= " << U*x << "\n";
  cout << "D^3 x^2\t= " << (U*U*U)*(x*x) << "\n";
  cout << "K L\t= " << K*L << "\n";
  cout << "D*\t= " << adjoint (U) << "\n";
  cout << "(D+x)*\t= " << adjoint (U+x) << "\n";
  cout << "(x D)*\t= " << adjoint (x*U) << "\n";
  cout << "K*\t= " << adjoint (K) << "\n";
  cout << "L*\t= " << adjoint (L) << "\n";
  cout << "K**\t= " << adjoint (adjoint (K)) << "\n";
  cout << "L**\t= " << adjoint (adjoint (L)) << "\n";
  cout << "K...\t= " << iterate (K) << "\n\n";

  cout << "K quo (D+x)\t= " << quo (K, U+x) << "\n";
  cout << "K rem (D+x)\t= " << rem (K, U+x) << "\n";
  cout << "K*K quo L\t= " << quo (K*K, L) << "\n";
  cout << "K*K rem L\t= " << rem (K*K, L) << "\n\n";

  for (int i=1; i<=7; i++)
    cout << "(D+x)^" << i << "\t= " << binpow (U+x, i) << "\n";

  cout << "\n";
  test (K, U+x);
  test (K*K, L);
  test (binpow (U+x, 20), L*L);
  cout << binpow (U+x, 15) << "\n";
  cout << quo (binpow (U+x, 15), L*L) << "\n";

  return 0;
}
