
/******************************************************************************
* MODULE     : mmx_gmp.cpp
* DESCRIPTION: Additional code for interface to gmp and mpfr
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "mmx_mpfr.hpp"
#include "integer.hpp"
#include "string.hpp"
namespace mmx {

nat        mmx_precision    = 64;
mpfr_rnd_t mmx_rounding_mode= GMP_RNDN;

string
mpfr_to_string (const mpz_t x) {
  char* s= mpz_get_str (NULL, 10, x);
  string r= s;
  mpfr_free_str (s);
  return r;
}

string
mpfr_to_string (const mpfr_t x, nat bits) {
  string r;
  if (mpfr_nan_p (x) != 0) return "NaN";
  if (mpfr_inf_p (x) != 0) {
    if (mpfr_sgn (x) < 0) return "-Infty";
    return "Infty";
  }
  nat digs= bits==0? 0: max (((int) (0.301029995 * bits)) - 1, 2);
  mp_exp_t exponent;
  char* mantissa_s= mpfr_get_str (NULL, &exponent, 10, digs, x, GMP_RNDN);
  int first= 0;
  if (mantissa_s[0] == '-') { r << "-"; first++; }
  r << mantissa_s[first];
  r << ".";
  r << (mantissa_s + (first + 1));
  r << "e" << as_string (exponent-1);
  mpfr_free_str (mantissa_s);
  return r;
}

string
zero_to_string (const mpfr_t err) {
  string r;
  mp_exp_t exponent;
  char* mantissa_s= mpfr_get_str (NULL, &exponent, 10, 2, err, GMP_RNDN);
  r << "0.0e" << as_string (exponent-1);
  mpfr_free_str (mantissa_s);
  return r;
}

ostream&
operator << (ostream& out, const mpfr_t x) {
  return out << mpfr_to_string (x, 0);
}

string
as_string (const integer& i) {
  char* s= mpz_get_str (NULL, 10, *i);
  string r= s;
  mpfr_free_str (s);
  return r;
}

} // namespace mmx
