
/******************************************************************************
* MODULE     : big_product.cpp
* DESCRIPTION: Big commutative products
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "big_sum.hpp"
#include "big_product.hpp"
#include "generic_internal.hpp"
#include "list_sort.hpp"
namespace mmx {
#define Table table<generic,generic>
#define Iterator iterator<generic>

/******************************************************************************
* Routines for big products
******************************************************************************/

generic
flatten (const big_product& p) {
  table<generic,generic> prod= *p;
  list<generic> l (entries (prod));
  l= sort (l, small_big_compare);
  generic acc= 1;
  while (!nil (l)) {
    generic mon  = flatten (car (l));
    generic coeff= flatten (prod[car (l)]);
    bool old= set_frac_flag (true);
    acc= acc * pow (mon, coeff);
    (void) set_frac_flag (old);
    l= cdr (l);
  }
  return acc;
}

Table
product_to_table (const generic& x) {
  if (is<big_product> (x))
    return *as<big_product> (x);
  if (is<big_sum> (x)) {
    Table sum= *as<big_sum> (x);
    if (N (sum) == 1) {
      generic y= *entries (sum);
      Table t (0);
      if (is<big_product> (y)) t= copy (*as<big_product> (y));
      else t (y)= 1;
      t (sum [y])= 1;
      return t;
    }
  }
  Table t (0);
  if (x != 1) t (x)= 1;
  return t;
}

generic
table_to_product (const Table& t) {
  generic sc= 1;
  Table r (0);
  for (Iterator it= entries (t); busy (it); ++it) {
    generic x= *it;
    if (t[x] == 0);
    else if (is_scalar (x)) {
      if (t[x] == 1) sc= sc * x;
      else if (t[x] == -1) sc= sc / x;
      else r(x)= t[x];
    }
    else r(x)= t[x];
  }

  if (N(r) == 0) return sc;
  if (N(r) == 1) {
    generic x= *entries (t);
    if (t[x] == 1) {
      if (sc == 1) return x;
      Table u (0);
      u(x)= sc;
      return as<generic> (big_sum (u));
    }
  }
  if (sc == 1) return as<generic> (big_product (r));
  Table sum (0);
  sum (as<generic> (big_product (r)))= sc;
  return as<generic> (big_sum (sum));
}

#undef Table
#undef Iterator
} // namespace mmx
