
/******************************************************************************
* MODULE     : list_glue.cpp
* DESCRIPTION: Glue for lists
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_LIST_GLUE_HPP
#define __MMX_LIST_GLUE_HPP
#include "list.hpp"
#include "category_glue.hpp"
#include "tuple.hpp"
#include "alias.hpp"
#include "integer.hpp"
namespace mmx {
#define TMPL template<typename C>
#define List list<C>

TMPL List
make_list (const tuple<C>& t) {
  nat i, n= N (t);
  List r;
  for (i=n-1; i>0; i--)
    r= cons (t[i], r);
  return r;
}

TMPL
class list_element_rep: public alias_rep<C> {
  alias<List > a;
  nat i;
public:
  inline list_element_rep (const alias<List >& a2, nat i2):
    a (a2), i (i2) {}
  C read () const { return read_alias (a) [i]; }
  C& open () const { return open_alias (a) (i); }
  void close () const { close_alias (a); }
};

TMPL inline alias<C>
list_element (const alias<List >& a, const integer& i) {
  return new list_element_rep<C> (a, as_int (i));
}

TMPL inline C
list_access (const List& l, const integer& i) {
  return l[as_int (i)];
}

TMPL inline List list_cons (const C& x, const List& l) { return cons (x, l); }
TMPL inline C list_car (const List& l) { return car (l); }
TMPL inline List list_cdr (const List& l) { return cdr (l); }
TMPL inline bool list_nil (const List& l) { return nil (l); }
TMPL inline bool list_atom (const List& l) { return atom (l); }
TMPL inline List list_reverse (const List& l) { return reverse (l); }
TMPL inline integer list_length (const List& l) { return N (l); }

template<typename Cond, typename C> void
define_list () {
  define_type<Cond,List > (gen ("List", type_name<C> ()));
  define<Cond> ("list", make_list<C>);
  define_from_iterator<Cond,C> ("list");
  define_binary<Cond,List,mul_op> ();
  define<Cond> ("cons", list_cons<C>);
  define<Cond> ("car", list_car<C>);
  define<Cond> ("cdr", list_cdr<C>);
  define<Cond> ("nil?", list_nil<C>);
  define<Cond> ("atom?", list_atom<C>);
  define<Cond> ("reverse", list_reverse<C>);
  define<Cond> (MMX_SIZE, list_length<C>);
  define<Cond> (MMX_ACCESS, list_access<C>);
  define<Cond> (MMX_ACCESS, list_element<C>);
  define_downgrader<Cond,iterator<C>,List > ("generate", iterate<C>);
}

#undef TMPL
#undef List
} // namespace mmx
#endif // __MMX_LIST_GLUE_HPP
