
/******************************************************************************
* MODULE     : string_escape.cpp
* DESCRIPTION: Handling escape characters
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "string.hpp"
namespace mmx {

string
escape (const string& s) {
  int i, n= N(s);
  string r;
  for (i=0; i<n; i++)
    if ((s[i] == '\\') || (s[i] == '\"')) r << '\\' << s[i];
    else if (s[i] == '\b') r << "\\b";
    else if (s[i] == '\t') r << "\\t";
    else if (s[i] == '\n') r << "\\n";
    else r << s[i];
  return r;
}

string
unescape (const string& s) {
  int i, n= N(s);
  string r;
  for (i=0; i<n; i++)
    if ((i+1<n) && (s[i] == '\\')) {
      i++;
      if (s[i] == 't') r << '\t';
      else if (s[i] == 'n') r << '\n';
      else if (s[i] == 'b') {
	if (N(r)>0)
	  r.rep->resize (N(r)-1);
      }
      else r << s[i];
    }
    else r << s[i];
  return r;
}

string
quote (const string& s) {
  return "\"" * escape (s) * "\"";
}

string
unquote (const string& s) {
  if ((N(s) > 2) && (s[0] == '\"') && (s[N(s)-1] == '\"'))
    return unescape (s (1, N(s)-1));
  else if (starts (s, "/\"") && ends (s, "\"/"))
    return s (2, N(s)-2);
  else return unescape (s);
}

} // namespace mmx
