
/******************************************************************************
* MODULE     : glue.cpp
* DESCRIPTION: Routines for defining glue
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "category_glue.hpp"
#include "tuple.hpp"
#include "literal.hpp"
#include "compound.hpp"
#include "integer.hpp"
namespace mmx {

/******************************************************************************
* These definitions should be made before anything else
******************************************************************************/

bool
fall_back_equal (const generic& x, const generic& y) {
  return eq (x, y);
}

static bool
fall_back_unequal (const generic& x, const generic& y) {
  return neq (x, y);
}

void
define_prerequisites () {
  define_type<always,bool> ("Boolean");
  define_type<always,generic> ("Generic");
  define_type<always,literal> ("Literal");
  define_type<always,compound> ("Compound");
  define_type<always,routine> ("Function");
  define_type<always,integer> ("Integer"); // FIXME: why put this here?
  define_constant<always,bool> ("false", false);
  define_constant<always,bool> ("true", true);
  define<always,bool,generic,generic> (MMX_EQUAL, fall_back_equal);
  define<always,bool,generic,generic> (MMX_UNEQUAL, fall_back_unequal);
}

/******************************************************************************
* Gluing new data types
******************************************************************************/

array<generic>
all_type_names () {
  generic ret;
  if (current_ev->read (gen ("all_types"), ret))
    return as<array<generic> > (ret);
  return array<generic> ();
}

void
define_type_sub (const generic& name, nat id) {
  current_ev->set (gen ("type_name", as<generic> (id)), name);
  current_ev->set (gen ("type_id", name), as<generic> (id));
  if (!is_tuple_type (id) && !is_alias_type (id)) {
    array<generic> all_types= all_type_names ();
    all_types << name;
    current_ev->set (gen ("all_types"), as<generic> (all_types));
  }
}

nat
type_id (const generic& name) {
  generic id;
  if (current_ev->read (gen ("type_id", name), id) && is<nat> (id))
    return as<nat> (id);
  else return 1;
}

generic
type_name (nat id) {
  generic name;
  if (current_ev->read (gen ("type_name", as<generic> (id)), name))
    return name;
  else return "Unknown";
}

} // namespace mmx
