/******************************************************************************
* MODULE     : math_printer.cpp
* DESCRIPTION: Printing math expressions for TeXmacs
* COPYRIGHT  : (C) 2007  Gregoire Lecerf
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "string.hpp"
#include "table.hpp"
#include "generic.hpp"

namespace mmx {

static void
init_math_symbol_table (table<generic, generic, eq_op>& t) {
  t(MMX_ACCESS) = "_"; // or "%sqaccess"
  t(MMX_AND) = "/\\";
  t(MMX_CATALAN) = "%mathcatalan";
  t(MMX_COMPOSE) = "%circ";
  t(MMX_DELTA) = "%delta";
  t(MMX_DERIVATIVE) = "D";
  t(MMX_DOT) = "%dotaccess";
  t(MMX_DIV) = "div";
  t(MMX_E) = "%mathe";
  t(MMX_EQUAL) = "=";
  t(MMX_EQUIV) = "<=>";
  t(MMX_EULER) = "%matheuler";
  t(MMX_FACTORIAL) = "factorial";
  t(MMX_GTR) = ">";
  t(MMX_GTREQ) = ">=";
  t(MMX_GTRGTR) = ">>";
  t(MMX_I) = "%mathi";
  t(MMX_IMPLIES) = "=>";
  t(MMX_IN) = "%in";
  t(MMX_INFINITY) = "%infty";
  t(MMX_INTO) = "->";
  t(MMX_LESS) = "<";
  t(MMX_LESSEQ) = "<=";
  t(MMX_LESSLESS) = "<<";
  t(MMX_MAPSTO) = ":->";
  t(MMX_MINUS) = "-";
  t(MMX_MOD) = "mod";
  t(MMX_NOT) = "!";
  t(MMX_OR) = "\\/";
  t(MMX_OVER) = "/";
  t(MMX_PARTIAL) = "%partial";
  t(MMX_PI) = "%mathpi";
  t(MMX_PLUS) = "+&";
  t(MMX_POWER) = "^";
  t(MMX_PRIME) = "%prime";
  t(MMX_RANGE) = "%colons"; // problem with iterator flattening
  t(MMX_RANGEEQ) = "%ldots";
  t(MMX_ROW) = "row";
  t(MMX_SEQAND) = "and";
  t(MMX_SEQOR) = "or";
  t(MMX_SIZE) = "%card";
  t(MMX_SQTUPLE) = "list";
  t(MMX_TIMES) = "*&";
  t(MMX_TRANSTYPE) = ":>";
  t(MMX_TUPLE) = "tuple";
  //t(MMX_TYPE) = ":";
  t(MMX_UNEQUAL) = "!=";
  t(MMX_VWHERE) = "|";
  t(MMX_WHERE) = "||";
  t(MMX_XOR) = "%veebar";

  t("alpha") = "%alpha";
  t("beta") = "%beta";
  t("gamma") = "%gamma";
  t("delta") = "%delta";
  t("epsilon") = "%varepsilon";
  t("zeta") = "%zeta";
  t("eta") = "%eta";
  t("theta") = "%theta";
  t("iota") = "%iota";
  t("kappa") = "%kappa";
  t("lambda") = "%lambda";
  t("mu") = "%mu";
  t("nu") = "%nu";
  t("xi") = "%xi";
  t("omicron") = "%omicron";
  t("pi") = "%pi";
  t("rho") = "%rho";
  t("sigma") = "%sigma";
  t("tau") = "%tau";
  t("upsilon") = "%upsilon";
  t("phi") = "%varphi";
  t("chi") = "%chi";
  t("psi") = "%psi";
  t("omega") = "%omega";

  t("Alpha") = "%Alpha";
  t("Beta") = "%Beta";
  t("Gamma") = "%Gamma";
  t("Delta") = "%Delta";
  t("Epsilon") = "%Epsilon";
  t("Zeta") = "%Zeta";
  t("Eta") = "%Eta";
  t("Theta") = "%Theta";
  t("Iota") = "%Iota";
  t("Kappa") = "%Kappa";
  t("Lambda") = "%Lambda";
  t("Mu") = "%Mu";
  t("Nu") = "%Nu";
  t("Xi") = "%Xi";
  t("Omicron") = "%Omicron";
  t("Pi") = "%Pi";
  t("Rho") = "%Rho";
  t("Sigma") = "%Sigma";
  t("Tau") = "%Tau";
  t("Upsilon") = "%Upsilon";
  t("Phi") = "%Phi";
  t("Chi") = "%Chi";
  t("Psi") = "%Psi";
  t("Omega") = "%Omega";
}

void
math_print (ostream& out, const generic& g) {
  static table<generic, generic, eq_op> math_symbol_table;
  if (N(math_symbol_table)==0)
    init_math_symbol_table (math_symbol_table);

  generic tmp;
  if (math_symbol_table->read (g, tmp)) out << literal_to_string (tmp);
  else if (is<literal> (g)) out << literal_to_string (g);
  else if (is<compound> (g)) {
    nat i, n= N (g);
    out << "(";
    if (n>0) {
      if (eq (g[0], MMX_SQTUPLE) && n>1 &&
	  is<compound> (g[1]) && eq (g[1][0], MMX_ROW))
	out << "matrix";
      else math_print (out, g[0]);
    }
    for (i=1; i<n; i++) {
      out << " ";
      math_print (out, g[i]);
    }
    out << ")";
  }
  else {
    select_output_evaluator ();
    generic f= flatten (g);
    restore_evaluator ();
    math_print (out, f);
  }
}

} // namespace mmx
